/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package CatCore.Arrows;

import CatCore.CTO;
import CatCore.ConformanceError;
import CatCore.Domains.Domain;
import CatCore.ExecutionError;
import CatCore.Main;
import CatCore.Var;
import CatGui.MainGui;
import java.io.FileNotFoundException;
import java.io.PrintStream;

/**
 *
 * @author dsb
 */
public class Arrow extends AorP {

    protected String name;
    protected String[] domains;
    protected String[] ranges;

    // used only for debugging
    public Arrow() {
    }

    // each arrow in a category is registered by invoking this constructor
    public Arrow(String name, String domain, String range) {
        this.name = name;
        this.domains = parseObject(domain, name);
        this.ranges = parseObject(range, name);
        // leaking constructor
        AorP.enter(name, this);
    }

    // -----------------general Arrow methods listed below----------------------
    // given an arrow, invoke execute() to execute it  -- details are arrow specific.
    // Arrow should be abstract, but there are too many concrete methods to test.
    @Override
    public void execute() throws ExecutionError, ConformanceError {
        Main.error("Arrow.execute is called -- should always be overrridden!");
    }

    @Override
    public String name() {
        return name;
    }

    public String[] domains() {
        return domains;
    }

    public String[] ranges() {
        return ranges;
    }

    @Override
    public String toString() {
        String path = name + ":";
        String times = "";
        for (String i : domains) {
            path = path + times + i;
            times = "*";
        }
        times = "->";
        for (String j : ranges) {
            path = path + times + j;
            times = "*";
        }
        return path;
    }

    // ensure that the arrow and each of its domains are present
    @Override
    public void validate(PrintStream ps) {
        AorP a = AorP.get(name);
        if (a == null || !(a instanceof Arrow)) {
            ps.println("arrow not in dictionary: " + name);
        }
        subvalidate(domains, ps);
        subvalidate(ranges, ps);
        unique(ps);
    }

    private void subvalidate(String[] array, PrintStream ps) {
        for (String i : array) {
            if (i.contains("_")) {
                i = Domain.getBaseName(i);
            }
            Domain d = Domain.get(i);
            if (d == null) {
                ps.println("undefined domain " + i);
            }
        }
    }

    private void unique(PrintStream ps) {
        // form single array that is the concatenation of array1 and array2
        String[] array = new String[domains.length + ranges.length];
        System.arraycopy(domains, 0, array, 0, domains.length);
        array[array.length - 1] = ranges[0];

        // now check to see if all elements of this array are unique
        for (int i = 0; i < array.length; i++) {
            for (int j = i + 1; j < array.length; j++) {
                if (i == j) {
                    continue;
                }
                if (array[i].equals(array[j])) {
                    ps.format("arrow %s has multiple copies of a domain %s as arguments or result", name, array[i]);
                }
            }
        }
    }

    //------------------------Utilities----------------------------------------
    // an "object" is the cross product of one or more domain names: name1 * name2 * ....
    // note: this method really should be private, but as it has tests, 
    // I had to makeReplica it public
    public static String[] parseObject(String o, String name) {
        // Step 1: clean up the object by removing all blanks
        String oo = o.replace(" ", "");

        // Step 2: split it by "*"
        String[] domains = oo.split("\\*");

        for (String domain : domains) {
            if (domain.equals("") || domain.startsWith("_")) {
                Main.error("empty domain in arrow " + name);
            }
        }
        return domains;
    }

    public void copyfile(String in, String out) {
        PrintStream ps;
        try {
            ps = new PrintStream(out);
            addOn(in, ps);
            ps.flush();
            ps.close();
        } catch (FileNotFoundException e) {
            Main.error("unable to copy file " + in + "\n" + e.getMessage());
        }
    }
    
    // for gui consumption
    
    protected void computePartialIoArray() {
        String[] io;
        String[] objs;
        CTO cto;
        int size;
        int i;
        size = domains().length;
        io = new String[size + 3];
        io[0] = Main.GVIM_EXE;
        io[1] = "-O" + (size+1);
        objs = domains();
        for (i = 0; i < size; i++) {
            cto = Var.get(objs[i]);
            io[i + 2] = cto.fullName();
        }
        CatGui.MainGui.io = io;
    }


    // for gui consumption
    protected void setSchemaAndConform() {
        String r = ranges[0];
        Domain d = Domain.get(r);
        if (d.ext().equals("pl")) {  // prolog-producing arrow
            CatGui.MainGui.targetSchema = CatCore.Main.METAMODEL_PATH + "/libpl/" + r + ".schema.pl";
            CatGui.MainGui.targetConform = CatCore.Main.METAMODEL_PATH + "/libpl/" + r + ".conform.pl";
        } else {
            clearSchemaAndConform();
        }
    }

    // -- needed to run the measly tests for CatCore
    @Override
    public void startComputingIOArray() {
        throw new UnsupportedOperationException("only subclassed method should be called"); 
    }
    
    // general operation.  It is the startComputingIOArray 
    // that is arrow specific
    @Override
    public void finishComputingIOArray() {
        String[] objs;
        CTO cto;
        if (MainGui.io != null) {
            objs = ranges();
            cto = Var.get(objs[0]);
            // if there was an error in computing the output, cto is null
            if (cto == null) {
                MainGui.io[MainGui.io.length - 1] = CatCore.Main.errorMsgFile();
            }
            else {
                MainGui.io[MainGui.io.length - 1] = cto.fullName();
            }
        }
    }
}
