/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package CatCore.Arrows;

import CatCore.CTO;
import CatCore.ConformanceError;
import CatCore.Domains.Domain;
import CatCore.ExeMethods;
import CatCore.ExecutionError;
import CatCore.Var;
import java.io.File;
import java.io.PrintStream;

/**
 *
 * @author dsb
 */
public class ArrowExe extends ArrowPrologUtilities {

    // an executable maps 1+ input files to a single file.  
    // exePath is the path to the executable.
    String exePath;

    public ArrowExe(String name, String domainName, String rangeName, String exePath ) {
        // Step 1: declare arrow itself and the 'executable' vm template
        super(name, domainName, rangeName);
        this.exePath = exePath;
    }

    @Override
    public void execute() throws ExecutionError, ConformanceError {
        int index;
        CTO in,first;
        
        // Step 1: create a command line array of the form
        //  (executable) (input1) ... (inputn) (output)
        //  note that the output name is based on the first input file
        String[] cmd = new String[domains.length + 2];
        cmd[0] = exePath;
        index = 1;
        for (String d : domains) {
            in = Var.get(d, this);
            cmd[index++] = CatCore.Main.normalizePath(in.fullName());
        }
        first = Var.get(domains[0], this);
        Domain out = Domain.get(ranges[0]);
        CTO o = out.computeNewCTOGivenCTO(first);
        cmd[index++] = CatCore.Main.normalizePath(o.fullName());
        
        // Step 2: execute the command -- result will be object o
        try {
            ExeMethods.executeCatCmdLine(cmd);
        } catch (ExecutionError | ConformanceError e) {
            System.err.format("CatLite halts\n");
            System.err.format("consult %s\n", CatCore.Main.errorMsgFile());
            throw new ExecutionError("C1: "+ e.getMessage());
        }
        
        // Step 3: Enter the newly created file into Var and check for conformance
        Var.put(o);
        o.conform();
    }
    
    // ensure that there is a vm file to read
    @Override
    public void validate(PrintStream eps) {
        super.validate(eps);
        File f = new File(exePath);
        if (!f.exists())
            eps.println("cannot find executable "+exePath);
    }
    
    @Override
    public void startComputingIOArray() {
        clearSchemaAndConform();
        CatGui.MainGui.arrowFile = null; // no vm or pl file to edit
        computePartialIoArray();
    }
        
}