/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package CatCore.Arrows;

import CatCore.CTO;
import CatCore.ConformanceError;
import CatCore.Domains.Domain;
import CatCore.ExeMethods;
import CatCore.ExecutionError;
import CatCore.Main;
import CatCore.Var;
import java.io.File;
import java.io.PrintStream;

/**
 *
 * @author dsb
 */
public class ArrowJava extends Arrow {

    // A java arrow maps n>=1 files to precisely 1 file
    String javaExe; // path to Java file to execute

    //--------------------------------------------------------
    // utilities for processing a prolog arrow
    // these methods pack multiple prolog files into a single prolog
    // file (returned as a CTO)
    public ArrowJava(String name, String domain, String range, String javaExe) {
        super(name, domain, range);
        this.javaExe = javaExe;
    }

    // executes java arrow.  n>=1 files are input; precisely 1 file is output
    // Var contains the input and output files
    @Override
    public void execute() throws ExecutionError, ConformanceError {
        CTO in, first;
        int index;

        File ef = new File(CatCore.Main.errorMsgFile());
        if (ef.exists()) {
            ef.delete();
        }

        // Step 1: create the command line string of file fileInputs (fin)
        String[] fileInputs = new String[domains.length + 1];
        index = 0;
        for (String d : domains) {
            in = Var.get(d, this);
            fileInputs[index++] = CatCore.Main.normalizePath(in.fullName());
        }

        // Step 2: create the command line string of the file output (fout) and
        //         produce the lone output CTO -- whose name is based on the 
        //         first input
        first = Var.get(domains[0], this);
        Domain out = Domain.get(ranges[0]);
        CTO o = out.computeNewCTOGivenCTO(first);
        fileInputs[index++] = CatCore.Main.normalizePath(o.fullName());

        // Step 3: create the command line to execute
        /* old way was to call java executable directly -- this code works 
         String[] exearray = {};  // here's where you put in command-line options -- I assume none
         String[] cmdarray = concat(exearray, fileInputs);

         try {
         c = Class.forName(javaExe);
         } catch (Exception e) {
         System.err.println("CatLite halts -- Java program failed execution");
         System.err.println("could not find : " + javaExe);
         System.err.println(e.getMessage());
         throw new ExecutionError(e);
         }
         try {
         Object i = c.newInstance();  // not really needed but..
         Method m = c.getMethod("main", String[].class);
         m.invoke(i, (Object) cmdarray);
         } 
         catch (InstantiationException | InvocationTargetException | IllegalAccessException | NoSuchMethodException e1) {
         System.err.println(e1.getClass().getName() + " exception");
         gracefulExit(e1);
         }

         catch (RuntimeException e1) {
         gracefulExit(e1);
         }
         --------------------------end of old code ------*/ //"cmd", "/c", "start", 
        String[] firstargs = {Main.JAVA_EXE, javaExe};
        String[] cmdarray = concat(firstargs, fileInputs);

        ExeMethods.executeCatCmdLine(cmdarray);
        
        File e = new File(CatCore.Main.errorMsgFile());
        if (e.length() != 0) {
            gracefulExit();
        }

        // Step 4: Enter the newly created file into Var and check for conformance
        Var.put(o);
        o.conform();
    }

    private void gracefulExit() {
        System.err.format("CatCore halts -- Java program (%s) failed execution\n", javaExe);
        System.err.format("consult %s for more information.\n", CatCore.Main.errorMsgFile());
        throw new ExecutionError("C0: CatCore Halts");
    }

    // produce an array of strings that is formed by
    // a concatenation of a set of string arrays
    public String[] concat(String[]  
        ... list) {

        int size = 0;
        for (String[] i : list) {
            size += i.length;
        }

        String[] result = new String[size];
        int index = 0;
        for (String[] j : list) {
            for (String k : j) {
                result[index++] = k;
            }
        }

        return result;
    }

    // confirm that there is indeed such an executable
    @Override
    public void validate(PrintStream ps) {
        super.validate(ps);
//        CatCore.Main.printClassPath("in validate");
        try {
            Class c = Class.forName(javaExe);
        } catch (ClassNotFoundException e) {
            ps.println("cannot find java executable: " + javaExe);
        }
    }

    @Override
    public void startComputingIOArray() {
        setSchemaAndConform();
        CatGui.MainGui.arrowFile = null; // no vm or pl file to edit
        computePartialIoArray();
    }
}
