/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package CatCore.Arrows;

import CatCore.ConformanceError;
import CatCore.Domains.Domain;
import CatCore.ExecutionError;
import CatCore.Main;
import java.io.PrintStream;

/**
 *
 * @author dsb
 */
public class ArrowPath extends AorP {

    String name;
    String[] body;
    String   path;  // used for gui

    public ArrowPath(String name, String path) {
        this.name = name;
        this.body = PathParse.parseBody(path);
        this.path = path;
        // leaking constructor -- needed for consistency
        AorP.enter(name, this);
    }
    
    public ArrowPath(String line) { // ad hoc arrows that don't end in ';' 
        this.name = "ad hoc";
        this.body = PathParse.partition(line);
        this.path = line;
    }

    public static ArrowPath make(String line) {
        PathParse p = new PathParse(line);
        return p.parse();
    }
    
    public String path() {
        return path;
    }

    @Override
    public String toString() {
        String b = body[0];
        for (int i = 1; i < body.length; i++) {
            b = b + "." + body[i];
        }
        return name + " = " + b + ";";
    }

    @Override
    public void execute() throws ExecutionError, ConformanceError {
        String errMsg = "in path " + name;
        AorP a;
        for (String an : body) {
            a = AorP.get(an);
            if (a == null) {
                // not defined -- must be an rename or move
                a = ArrowPath.returnAdHocArrow(an, errMsg);
            }
            a.execute();
        }
    }

    // this method registers ad hoc arrows in a pth
    @Override
    public void validate(PrintStream ps) {
        String result = validate();
        if (result.equals(""))
            return;
        ps.println(result);
    }
    
    // consider every arrow (recursively) within a pth.
    // if the arrow is recognizable, great.  If not, collect
    // a set of errors to report 
    public String validate() {
        String result = "";
        AorP a;

        for (String i : body) {
            a = AorP.get(i);
            if (a == null) {
                boolean good = i.contains(">") || i.contains("+") || i.contains("?");
                if (!good) {
                    result = result + "unrecognizable arrow " + i + " in path " + name + "\n";
                }
            }
            if (a instanceof ArrowPath) {
                result = result + ((ArrowPath) a).validate();
            }
        }
        return result;
    }

    // this method checks to see if domain d exists.  arrow a is used only for error 
    // reporting.  If the base domain does not exist, we have an error.
    // if the replica domain does not exist, we will create it.
    public static void doesDomainExist(String d, String a) {
        String baseDomainName = Domain.getBaseName(d);
        Domain aDomain = Domain.get(baseDomainName);
        if (aDomain == null) {
            Main.error("unrecognizable domain " + d + " in move arrow " + a);
        }
        if (Domain.get(d) == null) {
            Domain.makeReplica(d);
        }
    }

    @Override
    public String name() {
        return name;
    }

    // 2 kinds of Ad hoc arrows:  Move  domain1>domain2
    //                                  where both domains should have the same base
    enum AdHocArrowType {
        move, rename, conform, bad
    };

    public static Arrow returnAdHocArrow(String i, String pathName) {
        String[] parts;
        // must be a move arrow (which is "domain1>domain2") 
        // or a rename arrow "domain+string"
        // if not, we have an error
        AdHocArrowType at = i.contains(">") ? AdHocArrowType.move : i.contains("+")
                ? AdHocArrowType.rename : i.contains("?") ? AdHocArrowType.conform : AdHocArrowType.bad;
        switch (at) {
            case move:
                parts = i.split(">");
                if (parts.length != 2) {
                    break;
                }
                doesDomainExist(parts[0], i);
                doesDomainExist(parts[1], i);
                return new ArrowMove(i, parts[0], parts[1]);
            case rename:
                parts = i.split("\\+");
                if (parts.length != 2) {
                    break;
                }
                doesDomainExist(parts[0], i);
                return new ArrowRename(i, parts[0], parts[0], parts[1]);
            case conform:
                parts = i.split("\\?");
                if (parts.length != 1) {
                    break;
                }
                doesDomainExist(parts[0], i);
                return new ArrowConform(i, parts[0], parts[0]);
            case bad:
        }
        Main.error("unrecognizable arrow " + i + " in path " + pathName);
        return null;
    }
//        if (move) {
//            parts = i.split(">");
//            if (parts.length != 2) {
//                Main.error("unrecognizable arrow " + i + " in pth " + pathName);
//            }
//                
//            doesDomainExist(parts[0], i);
//            doesDomainExist(parts[1], i);
//            return new ArrowMove(i, parts[0], parts[1]);
//        }  if (rename) {
//        parts = i.split("\\+");
//        if (parts.length != 2) {
//            Main.error("unrecognizable arrow " + i + " in pth " + pathName);
//        }
//        doesDomainExist(parts[0], i);
//        return new ArrowRename(i, parts[0], parts[0], parts[1]);
//        } else if (conform) { // conform arrow
//            parts = i.split("?");
//            if (parts.length != 1)
//                Main.error("unrecognizable arrow " + i + " in pth " + pathName);
//            doesDomainExist(parts[0],i);
//            return new ArrowConform(i, parts[0], parts[0]);
//        } else {

    /*---------------- for GUI consumption --------------*/
    public AorP[] getPath() {
        AorP a;
        AorP[] pth = new AorP[body.length];
        int i = 0;
        for (String an : body) {
            a = AorP.get(an);
            if (a == null) {
                // must be ad hoc -- if there is no such arrow,
                // then an execution error is thrown
                a = ArrowPath.returnAdHocArrow(an, name);
            }
            pth[i++] = a;
        }
        return pth;
    }

    @Override
    public void startComputingIOArray() {
        clearSchemaAndConform();
        CatGui.MainGui.arrowFile = null;
        CatGui.MainGui.io = null;
    }

    @Override
    public void finishComputingIOArray() {
        throw new UnsupportedOperationException("Should Never Be Called"); //To change body of generated methods, choose Tools | Templates.
    }

}
