/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package CatCore.Arrows;

import CatCore.CTO;
import CatCore.ConformanceError;
import CatCore.Domains.Domain;
import CatCore.ExeMethods;
import CatCore.ExecutionError;
import CatCore.Main;
import CatCore.Var;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.PrintStream;

/**
 *
 * @author dsb
 */
public class ArrowProlog extends ArrowPrologUtilities {

    //---------------- instance definition
    String prologFile; // path to prolog file to execute

    //--------------------------------------------------------
    // utilities for processing a prolog arrow
    // these methods pack multiple prolog files into a single prolog
    // file (returned as a CTO)
    public ArrowProlog(String name, String domain, String range, String prologFile) {
        // Step 1: declare arrow in general and remember its prolog executable
        super(name, domain, range);
        this.prologFile = prologFile;

//       // now do some type checking to see if the ext() of all i/o domains is "pl"
//       checkTypeConstraints(domains,name,"prolog","pl","input");
//       checkTypeConstraints(ranges,name,"prolog","pl","output");
    }

    // shorthand for generating file paths
    private String libplm(String name) {
        return libpl(name, Main.METAMODEL_PATH);
    }

    private String libplc(String name) {
        return libpl(name, Main.CORE_PATH);
    }

    private String libpl(String name, String path) {
        String x = path + "libpl/" + name;
        if (x.endsWith(".pl")) {
            return x;
        } else {
            return x + ".pl";
        }
    }


    @Override
    public void execute() throws ExecutionError, ConformanceError {
        CTO in, input;
        PrintStream ps;
        String fileInputs[];
        int index;

        // Step 1: allocate fileinputs: discontiguous file + # of domain files + translationFile +
        //         schema to produce + printfile + schema run file
        fileInputs = new String[domains.length + 5];
        index = 0;
        fileInputs[index++] = libplc("discontiguous");
        for (String d : domains) {
            in = Var.get(d, this);
            fileInputs[index++] = in.fullName();
        }
        fileInputs[index++] = libplm(prologFile);
        fileInputs[index++] = libplm(ranges[0] + ".schema");
        fileInputs[index++] = libplc("print");
        fileInputs[index++] = libplc("run");

        input = mergeIntoX("% code to execute arrow " + name(), fileInputs, System.err);

        // Step 2: produce the lone output object -- name is based on first input
        CTO tmp = Var.get(domains[0], this);
        Domain out = Domain.get(ranges[0]);
        CTO o = out.computeNewCTOGivenCTO(tmp);

        // Step 2: now produce the script to execute
        try {
            ps = new PrintStream(Main.scriptFile());
            ps.print(":-['" + CatCore.Main.normalizePath(input.fullName()) + "'],tell('" + CatCore.Main.normalizePath(o.fullName()) + "'),run,told,halt.");
            ps.flush();
            ps.close();
        } catch (FileNotFoundException e) {
            System.err.println(e.getMessage());
            return;
        }

        // Step 3: create the command line to execute
        String[] cmdarray = {CatCore.Main.SWIPL_EXE, "--quiet", "-f", Main.scriptFile()};

        // fork a process to execute
        try {
            ExeMethods.executeCatCmdLine(cmdarray);
        } catch (ExecutionError | ConformanceError e) {
            System.err.format("CatLite halts -- SWI Prolog Errors detected\n");
            System.err.format("consult %s\n", CatCore.Main.errorMsgFile());
            System.err.format("debug this prolog file:  %s\n", input.fullName());
            System.err.println(e.getMessage());
            throw new ExecutionError("C1: "+ e.getMessage());
        }

        // Step 4: clean up -- delete the script file,  PLExeFile (if it was used) and enter the newly created file.
//        File s = new File(scriptFile);
//        s.delete();
//        s = new File(PLExeFile);
//        s.delete();
        // Step 5: Enter the newly created file into Var and check for conformance
        Var.put(o);
        o.conform();
    }

    // ensure that there is a prolog file to read
    @Override
    public void validate(PrintStream ps) {
        super.validate(ps);
        File f = new File(libplm(prologFile));
        if (!f.exists()) {
            ps.println("cannot find prolog file " + prologFile);
        }
    }

    //----------------for gui consumption---------------------
    @Override
    public void startComputingIOArray() {
        setSchemaAndConform();
        CatGui.MainGui.arrowFile = CatCore.Main.METAMODEL_PATH + "libpl/" + name() + ".pl";
        computePartialIoArray();
    }
}
