/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package CatCore.Arrows;

import CatCore.CTO;
import CatCore.Domains.Domain;
import CatCore.ExecutionError;
import CatCore.Main;
import CatCore.Var;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.PrintStream;

/**
 *
 * @author dsb
 */
abstract public class ArrowPrologUtilities extends Arrow {

    public ArrowPrologUtilities(String name, String domain, String range) {
        // relay constructor up the hierarchy
        super(name, domain, range);
    }

    // here's a common method -- an arrow takes 1+ prolog files as input
    // we need to return a single CTO -- it is either the lone file or 
    // it is a concatenation of all the prolog files into a single (manufactured)
    // CTO.  This is the method that does this task.
    public CTO getInputCTO(AorP arrow) {
        CTO input;

        // Step 1: get the input object 
        if (domains.length == 1) {
            // just use the input file as is
            input = Var.get(domains[0], arrow);
        } else {
            CTO inputs[] = new CTO[domains.length];
            for (int i = 0; i < domains.length; i++) {
                inputs[i] = Var.get(domains[i], arrow);
            }
            input = mergeCTOs("% code to implement arrow " + name, inputs, System.err);
        }
        return input;
    }

    // another common method -- used with getInputCTO
    // this method returns the CTO that is the output of an arrow
    public CTO getOutputCTO(AorP arrow) {

        // Step 1: get the output domain of the current arrow
        Domain od = Domain.get(ranges[0]);

        // Step 2: get the CTO of the first input domain
        CTO firstArg = Var.get(domains[0], arrow);

        // Step 3: the output object has the domain + ext information of od
        //         and the name of the first argument
        return new CTO(firstArg.name(), od.dom(), od.ext());
    }


    // the result of merged files is the new value of the given object
    // the CTO returned has a manufactured domain name XxX
    public CTO mergeCTOs(String commentHeader, CTO[] listOfCTOs, PrintStream eps) {
        return mergeCTOs(commentHeader, listOfCTOs, Main.PLExeFileName, Main.plDom, "pl", eps);
    }
    
    public CTO mergeCTOs(String commentHeader, CTO[] listOfCTOs, String fileName,
            String dom, String ext, PrintStream eps) {
        PrintStream ps;
        String outputFile;

        outputFile = fileName + "." + dom + "." + ext;
        try {
            File f = new File(outputFile);
            ps = new PrintStream(f);
            ps.format("%s\n\n", commentHeader);
            ps.format("refresh:-['%s'].\n\n:-", outputFile);
            for (CTO i : listOfCTOs) {
                ps.format("['%s'],",i.fullName());
            }
            ps.format("true.\n");
            ps.flush();
            ps.close();
        } catch (FileNotFoundException e) {
            throw new ExecutionError("C2: unable to open " + outputFile);
        }
        return CTO.make(outputFile);
    }

    public static CTO mergeIntoX(String commentHeader, String[] listOfFileNames, PrintStream eps) {
        return merge(commentHeader, listOfFileNames, Main.PLExeFile(), eps);
    }

    public static CTO mergeIntoC(String commentHeader, String[] listOfFileNames, PrintStream eps) {
        return merge(commentHeader, listOfFileNames, Main.PLConFile(), eps);
    }

    public static CTO merge(String commentHeader, String[] listOfFileNames, String outputFile, PrintStream eps) {
        PrintStream ps;
        try {
            File f = new File(outputFile);
            ps = new PrintStream(f);
            ps.format("%s\n\n", commentHeader);
            ps.format("refresh:-['%s'].\n\n:-", outputFile);
            for (String fileName : listOfFileNames) {
                ps.format("['%s'],", fileName);
            }
            ps.format("true.\n");
            ps.flush();
            ps.close();
        } catch (FileNotFoundException e) {
            throw new ExecutionError("C3: unable to open " + outputFile);
        }
        return CTO.make(outputFile);
    }
}
