/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package CatCore;

import CatCore.Arrows.ArrowPrologUtilities;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.PrintStream;
import java.util.Formatter;

/**
 *
 * @author dsb CTO = CatLiteObject -- a CTO is an object in a domain, and is
 * always paired with a file
 */
public class CTO implements java.io.Serializable {

    public static Process currentProcessExecuting = null;

    String filename;
    String domainName;
    String extName;

    public CTO(String filename, String domainName, String extName) {
        this.filename = filename;
        this.domainName = domainName;
        this.extName = extName;
    }

    public static CTO make(String fullPathName) {
        String[] parts = fullPathName.split("\\.");
        if (parts.length != 3) {
            Main.error("full path name has <file>.<domain>.<extension>, not " + fullPathName);
        }
        return new CTO(parts[0], parts[1], parts[2]);
    }

    protected CTO() {
    } // shouldn't be called

    public void rename(String newName) {
        File f = new File(filename);
        File n = new File(newName);
        if (f.renameTo(n)) {
            Main.error("unable to rename " + filename + "to " + newName);
        }
        filename = newName;
    }

    public String shortName() {
        int i = CatCore.Main.normalizePath(filename).lastIndexOf("/");
        if (i == -1) {
            return filename;
        } else {
            return filename.substring(i + 1);
        }
    }

    public String name() {
        return filename;
    }

    public String ext() {
        return extName;
    }

    public String dom() {
        return domainName;
    }

    public String fullName() {
        return CatCore.Main.normalizePath((filename + "." + dom() + "." + ext()));
    }

    public void delete() {
        File f = new File(fullName());
        f.delete();
    }

    public File toFile() {
        return new File(fullName());
    }

    // this method is used to test conformance of a prolog database
    // with its constraints.  it is coded only for this purpose
    // as it outputs nothing.
    public void conform() {
        File e = null;
        // Step 1: conform is applied, really, only to pl files.
        //         non-pl files must have an explicit arrow for conformance
        switch (extName) {
            case "pl":
                // Step 2: well, we have a .pl file -- invoke standard method
                //         for conformance.
                String conformFile = Main.METAMODEL_PATH + "libpl/" + dom() + ".conform.pl";
                String[] list = {Main.CORE_PATH + "libpl/discontiguous.pl",
                    CatCore.Main.normalizePath(fullName()),
                    Main.CORE_PATH + "libpl/conform.pl",
                    conformFile};
                CTO x = ArrowPrologUtilities.mergeIntoC("% code to evaluate conformance testing of " + fullName(), list, System.err);
                try {
                    PrintStream ps;
                    ps = new PrintStream(Main.scriptFile());
                    ps.print(":-['" + CatCore.Main.normalizePath(x.fullName()) + "'],tell('" + Main.conformMsgFile() + "'),run,told,halt.");
                    ps.flush();
                    ps.close();
                } catch (FileNotFoundException xx) {
                    Main.error(xx);
                }
                String[] cmdarray = {Main.SWIPL_EXE, "--quiet", "-f", Main.scriptFile()};
                try {
                    ExeMethods.executeCatCmdLine(cmdarray);
                } catch (ExecutionError | ConformanceError xx) {
                    String line1 = "CatLite halts -- SWI Prolog Errors detected";
                    String line2 = "consult " + Main.conformMsgFile();
                    String line3 = "debug this prolog file:   " + fullName();
                    String line4 = Main.conformanceFailure;
                    String line5 = "debug this prolog file:   " + fullName();

                    Formatter fm = new Formatter();
                    fm.format("%s\n%s\n%s\n%s\n%s\n", line1, line2, line3, line4, line5);
                    String msg = fm.toString();

                    throw new ConformanceError("C50: " + msg);
                }
                File c = new File(Main.conformMsgFile());
                if (c.length() != 0) {
                    String msg = Main.conformanceFailure + "\n" + "look at script.txt and conformance errors in "
                            + CatCore.Main.conformMsgFile();
                    throw new ConformanceError("C51: " + msg);
                }
                c.delete();
                File s = new File(Main.scriptFile());
                s.delete();
                break;
            case "bool":
                String boolFileName = fullName();
                try {
                    e = new File(boolFileName);
                    if (e.length() != 0) {
                        String msg = Main.conformanceFailure + "\n" + "look at errors in " + boolFileName;
                        throw new ConformanceError("C52: " + msg);
                    }
                } catch (ConformanceError xx) {
                    Main.error("Can't open " + boolFileName);
                }
                break;
        }
    }

    public static void stopProcess() {
        currentProcessExecuting.destroy();
    }

    public static void done(Exception e) {
        System.err.println(e.getMessage());
        throw new ExecutionError("C30:" + e.getMessage());
    }
}
