/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package CatCore;

import java.io.*;
import java.util.LinkedList;

/**
 *
 * @author dsb
 */
public class CheckEnvironment {

    public static void main(String[] args) {
        doCheck(args);
    }

    public static void doCheck(String[] args) {
        if (args.length == 3) {
            System.out.println("If you see this line, JAVA_EXE is correct.");
            return;
        } else if (args.length != 0) {
            throw new ExecutionError("C8: " + CheckEnvironment.class.getName()
                    + "called with incorrect number of arguments");
        }

        // normal invocation -- no arguments
        Main.readConfigFile();

        //  Step 1: check to see if prolog can be executed -- this also checks catlib setting
        String swipl = Main.SWIPL_EXE;
        String filename = "script.txt";

        System.out.print("Testing SWIPL_EXE and CATLIB ... ");

        try {
            PrintStream ps;
            ps = new PrintStream(filename);
            ps.print(":-['" + Main.CORE_PATH + "libpl/swiplInstalled'],run,halt.");
            ps.flush();
            ps.close();
        } catch (FileNotFoundException e) {
            Main.error(e);
        }
        String[] cmdarray = {swipl, "--quiet", "-f", "script.txt"};
        try {
            ExeMethods.executeCatCmdLine(cmdarray);
        } catch (ExecutionError | ConformanceError e) {
            System.err.println("something is wrong with Path, SWIPL_EXE, and/or CATLIB");
            System.err.println("SWIPL_EXE  = " + Main.SWIPL_EXE);
            System.err.println("CATLIB     = " + Main.CATLIB);
            System.err.println("PATH = " + System.getenv("Path"));
            System.err.println();
            System.err.println("run " + ExeMethods.executedLine + " which calls script.txt");
            System.err.println();
            return;
        }
        System.out.format("OK\n\n", (Object) null);

        // Step 2: now check out JAVA_EXE
        System.out.print("Testing JAVA_EXE ... ");
        String java_exe = Main.JAVA_EXE;
        String[] cmdarray2 = {java_exe, "CatCore.CheckEnvironment", "1", "2", "3"};
        try {
            ExeMethods.executeCatCmdLine(cmdarray2);
        } catch (ExecutionError | ConformanceError e) {
            System.err.println("Something is wrong with JAVA_EXE");
            return;
        }
        System.out.println("OK\n");

        // Step 3: now check out SWIPL-WIN_EXE
        System.out.print("Testing SWIPL_WIN_EXE ...  you should see a SWIPL_WIN window.  If so, close it.");

        String swiplwin = Main.SWIPL_WIN_EXE;
        String[] cmdarray3 = {"cmd", "/c", "start", swiplwin, Main.CORE_PATH + "libpl/swiplInstalled.pl"};
        try {
            ExeMethods.executeCatCmdLine(cmdarray3);
        } catch (ExecutionError | ConformanceError e) {
            System.err.println("cannot execute SWIPL_WIN_EXE");
            return;
        }
        System.out.format("OK\n\n", (Object) null);

        // Step 4: now check out GVIM_EXE 
        System.out.println("testing GVIM_EXE ... you should see a GVIM window.  If so, close it.");
        String vim = Main.GVIM_EXE;
        String[] cmdarray4 = {vim};
        try {
            ExeMethods.executeCatCmdLine(cmdarray4);
        } catch (ExecutionError | ConformanceError e) {
            System.err.println("cannot execute GVIM_EXE");
            return;
        }
        System.out.println();

        // Step 5: now check out BROWSER_EXE 
        String ibrowser = System.getenv("BROWSER_EXE");
        if (ibrowser == null) {
            System.out.println("warning: BROWSER_EXE not set");
        } else {
            System.out.println("testing BROWSER_EXE ... you should see a browser window.  If so, close it.");
            String[] cmdarray5 = {ibrowser};
            try {
                ExeMethods.executeCatCmdLine(cmdarray5);
            } catch (ExecutionError | ConformanceError e) {
                System.err.println("BROWSER error -- manually verify ");
                System.err.println(e.getMessage());
            }
        }
        System.out.format("%s", "\n\n");

        // Step 6: now check out violet
        System.out.println("testing violet ... you should see a browser window.  If so, close it.");
        String[] cmdarray6 = {"java", "com.horstmann.violet.UMLEditor"};
        try {
            ExeMethods.silentExe(cmdarray6);
        } catch (ExecutionError | ConformanceError e) {
            System.err.println("cannot execute violet: " + e.getMessage());
            return;
        }
        System.out.println();

        // Step 7: check if there are any missing jar files from CATLIB
        try {
            LinkedList<String> mylist = new LinkedList<>();
            recursiveFind(Main.CATLIB, mylist);
            
            String cp = System.getenv("CLASSPATH");
            boolean allPresent = true;
            
            for (String jar : mylist) {
                if (!cp.contains(jar)) {
                    System.err.format("jar file %s missing from CLASSPATH\n", jar);
                    allPresent = false;
                }
            }
            System.out.format("%s","\n\n");
            if (!allPresent) {
                return;
            }
        } catch (ConformanceError | ExecutionError | IOException e) {
            System.err.println("C36: error in checking presence of jar files " + e.getMessage());
            return;
        }
        System.out.println();

        System.out.println("CatCore is Ready to Use!");
        System.out.println("run CatCore.Clean to remove debugging/execution files");
    }

    public static void recursiveFind(String fileName, LinkedList<String> list) throws IOException {

        File file = new File(fileName);

        if (file.isDirectory()) {
            String files[] = file.list();
            for (String temp : files) {
                // if any are jar files -- add them to the list
                if (temp.endsWith(".jar")) {
                    list.add(temp);
                } else {
                    recursiveFind(fileName + "\\" + temp, list);
                }
            }
        }
    }
}
