/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package CatCore.Domains;

import CatCore.Main;
import CatCore.CTO;
import java.io.File;
import java.io.PrintStream;
import java.util.HashMap;

/**
 *
 * @author dsb
 */
public class Domain {
    // this is the static dictionary declaration

    private static HashMap<String, Domain> dictionary = new HashMap<>();

   // for state stacks
    public static HashMap<String, Domain> getContents() {
        return dictionary;
    }
    
    public static int size() {
        return dictionary.size();
    }

    public static void putContents(HashMap<String, Domain> x) {
        dictionary = x;
    }

    public static Domain get(String x) {
        return dictionary.get(x);
    }

    public static void init() {
        dictionary = new HashMap<>();
    }

    // for debugging
    static void dump() {
        System.out.println("dump of contents of Dom.dictionary");
        for (Domain i : dictionary.values()) {
            System.out.println(i);
        }
        System.out.println();
    }

    public static void validateAll(PrintStream ps, String modelHome) {
                
        //System.out.println(">>>" + modelHome);
        
        for (Domain d : dictionary.values()) {
            d.validate(ps, modelHome);
        }
    }

    public static String getBaseName(String x) {
        String y;
        int indx = x.indexOf("_");
        if (indx == -1) {
            y = x;
        } else {
            y = x.substring(0, indx);
        }
        return y;
    }
    //---------------------instance definition-------------------
    // This part is for defining instances of Domain: 
    // each domain has a string dom dn, and a string extension ex
    // a file of a domain has a string dom n.  Its true dom is 'n.dn.ex'
    // in the CatLite universe
    String dom;
    String ext;

    public Domain(String domName, String ext) {
        this.dom = domName;
        this.ext = ext;
        // leaking constructor
        dictionary.put(domName, this);
    }

    // this factory method defines domain replicas (domain_something)
    public static Domain makeReplica(String name) {
        if (name.indexOf('_') == -1) {
            Main.error("replica domain has no '_' char: " + name);
        }
        String base = Domain.getBaseName(name);
        Domain b = dictionary.get(base);
        if (b == null) {
            Main.error("replica domain has no base definition: " + name);
        }
        return new Domain(name, b.ext());
    }

    //-------------- Standard methods for DomainDefs-----------------
    public String dom() {
        return dom;
    }

    public String ext() {
        return ext;
    }

    // we want to compute the dom of a file for "this" domain given
    // an existing CTO
    public String computeNameGiven(CTO o) {
        return o.name() + dom() + ext();
    }

    public CTO computeNewCTOGivenCTO(CTO o) {
        return new CTO(o.name(), dom(), ext());
    }

    public void validate(PrintStream ps, String modelHome) {
        File f;
        
        // no matter what the metamodel, if there is a domain with a pl 
        // extension, then there needs to be a conformance and schema file
        if (ext().equals("pl")) {
           String schemaFile = modelHome + "libpl/" + dom() + ".schema.pl";
           f = new File(schemaFile);
           if(!f.exists())
               ps.println("prolog schema " + schemaFile.replace(modelHome,"") + " does not exist");
           
           String conformanceFile = modelHome + "libpl/" + dom() + ".conform.pl";
           f = new File(conformanceFile);
           if (!f.exists())
               ps.println("prolog conformance file " + conformanceFile.replace(modelHome,"") + " does not exist");
        }
    }
}
