/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package CatCore;

import CatCore.Arrows.AorP;
import CatCore.Arrows.ArrowJava;
import CatCore.Arrows.ArrowPath;
import CatCore.Arrows.ArrowProlog;
import CatCore.Arrows.ArrowVm;
import CatCore.Domains.Domain;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.PrintStream;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import javax.swing.JFileChooser;

/**
 *
 * @author dsb this is the CatLite interpreter
 */
public class Main {

    public static boolean debug = false;
    private static Properties configFile;
    static int errorCount;

    public static final String conformanceFailure = "Conformance Failure";

    public static final String conMsgFile = "conformance.txt";
    public static final String errMsgFile = "error.txt";  // where all arrow errors are printed
    public static final String scrFile = "script.txt";    // 

    public static final String PLConFileName = "Kon";
    public static final String PLExeFileName = "Exe";
    public static final String plDom = "X";

    public static String CATLIB = null;           // location of Categories directory
    public static String METAMODEL_PATH;          // location of metamodel cat.lite.pl, libpl and libvm files
    public static String CORE_PATH;               // location of Core libpl and libvm files
    public static String CURRENT_DIRECTORY;       // location of the current directory (where file read/write occurs)

    public static String JAVA_EXE;                // java.exe absolute path
    public static String GVIM_EXE;                // gvim.exe absolute path
    public static String SWIPL_EXE;               // swipl.exe absolute path
    public static String SWIPL_WIN_EXE;           // swipl-win.exe absolute path

    public static String ADVICE;                  // java program for metamodel advice
    public static String MY_HOME;                 // an optional metamodel identifier
    public static String SAVE_LIST;               // list of names whose matching files are not to be deleted

//    public static String CLASSPATH;               // the value of the classpath for the current execution environment
//    public static String PATH;                    // the value of the path variable for the current execution environment

    private static String override(String variable) {
        String value = System.getenv(variable);
        if (value != null) {
            configFile.put(variable, value);
        } 
        //else { configFile.put(variable, "");  }  // Catalina lite
        return (String) configFile.get(variable);
    }

    public static void readEnvironmentVariables() {
        

        // these are the values that should be set in the environment
        // whatever is unset, will be overridden by the config.properties file.
        CURRENT_DIRECTORY = GetAbsolutePath(new File("."));
        CATLIB = override("CATLIB");
        JAVA_EXE = override("JAVA_EXE");
        GVIM_EXE = override("GVIM_EXE");
        SWIPL_EXE = override("SWIPL_EXE");
        SWIPL_WIN_EXE = override("SWIPL_WIN_EXE");
        CORE_PATH = CATLIB + "/_Core";
        
        configFile.put("CATLIB",CATLIB);
        configFile.put("JAVA_EXE",JAVA_EXE);
        configFile.put("GVIM_EXE",GVIM_EXE);
        configFile.put("SWIPL_EXE",SWIPL_EXE);
        configFile.put("SWIPL_WIN_EXE",SWIPL_WIN_EXE);
        configFile.put("CORE_PATH",CORE_PATH);
        configFile.put("CURRENT_DIRECTORY", CURRENT_DIRECTORY);
    }

    public static void updateConfigFile(String msg) {
        try {
            PrintStream out = new PrintStream(CatCore.Main.CURRENT_DIRECTORY + "config.properties");
            DateFormat dateFormat = new SimpleDateFormat("yyyy/MM/dd HH:mm:ss");
            Date date = new Date();
            out.println(dateFormat.format(date));
            out.format("METAMODEL_PATH=%s\n", Main.METAMODEL_PATH);
            out.format("MY_HOME=%s\n", Main.MY_HOME);
            out.format("ADVICE=%s\n", Main.ADVICE);
            out.format("SAVE_LIST=%s\n", Main.SAVE_LIST);
            out.close();
//            FileOutputStream out = new FileOutputStream(CatCore.Main.CURRENT_DIRECTORY + "config.properties");
//            CatCore.Main.configFile.store(out, msg);
        } catch (FileNotFoundException e) {
            System.err.println("can't write config.properties file");
            System.err.println(e.getMessage());
        }
    }

    public static Properties cloneConfigFile() {
        return (Properties) CatCore.Main.configFile.clone();
    }

    public static void readConfigFile() {
        String tryThis = null;

        if (configFile != null) // initialize these files only once
        {
            return;
        }
        configFile = new Properties();

        // read local config.properties file.
        // if this fails, then read the config.properties file from 
        // the metamodel location and copy it to make it local
        String cp = null;
        try {
            cp = "config.properties";
            configFile.load(new FileInputStream(cp));
        } catch (IOException e) {
            File cd = null, sf;

            String catlib = System.getenv("CATLIB");
            if (catlib != null) {
                cd = new File(catlib);
                JFileChooser jfc = new JFileChooser(cd);
                jfc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
                jfc.setFileFilter(new MDEAppFilter());
                jfc.setDialogTitle("Which category to use?");
                int returnVal = jfc.showOpenDialog(null);
                if (returnVal == JFileChooser.APPROVE_OPTION) {
                    sf = jfc.getSelectedFile();
                    try {
                        METAMODEL_PATH = GetAbsolutePath(sf);
                        tryThis = Main.normalizePath(GetAbsolutePath(sf) + "/config.properties");
                        configFile.load(new FileInputStream(tryThis));
                        configFile.put("METAMODEL_PATH", METAMODEL_PATH);
                    } catch (IOException x) {
                        throw new ExecutionError("can't load config.properties file!");
                    }
                } else {
                    throw new ExecutionError("can't load config.properties file!");
                }
            } else {
                throw new ExecutionError("CATLIB environment variable is not set");
            }
        }

        readEnvironmentVariables();

        // ok -- we have read a properties file!  now set basic quantities
        CORE_PATH = dirNormalize("CORE_PATH");
        METAMODEL_PATH = dirNormalize("METAMODEL_PATH");
        CURRENT_DIRECTORY = dirNormalize("CURRENT_DIRECTORY");

        JAVA_EXE = exeNormalize("JAVA_EXE");
        GVIM_EXE = exeNormalize("GVIM_EXE");
        SWIPL_EXE = exeNormalize("SWIPL_EXE");
        SWIPL_WIN_EXE = exeNormalize("SWIPL_WIN_EXE");

        ADVICE = Main.configFile.getProperty("ADVICE");
        MY_HOME = Main.configFile.getProperty("MY_HOME");
        
        SAVE_LIST = Main.configFile.getProperty("SAVE_LIST");
        if (SAVE_LIST == null)
            SAVE_LIST = "";

        //System.out.println("MY_HOME = " + MY_HOME);
        if (tryThis != null) {
            updateConfigFile("copied from " + tryThis);
        }

//        CLASSPATH = System.getenv("CLASSPATH");
//        PATH = System.getenv("PATH");
    }
    
    public static String GetAbsolutePath(File f) {
        return f.getAbsolutePath().replace("\\.", "");
    }

    public static String conformMsgFile() {
        return CURRENT_DIRECTORY + conMsgFile;
    }

    public static String errorMsgFile() {
        return CURRENT_DIRECTORY + errMsgFile;
    }

    public static String scriptFile() {
        return CURRENT_DIRECTORY + scrFile;
    }

    public static String PLConFile() {
        return CURRENT_DIRECTORY + PLConFileName + "." + plDom + ".pl";
    }

    public static String PLExeFile() {
        return CURRENT_DIRECTORY + PLExeFileName + "." + plDom + ".pl";
    }

    public static String normalizePath(String path) {
        return path.replace("\\", "/").replace("\\:", ":");
    }

    static String dirNormalize(String whichPath) {
        String path = configFile.getProperty(whichPath);
        if (path == null) {
            throw new ExecutionError("C10: " + whichPath + " is not set in config.properties file");
        }
        //normalize name by (1) replacing '\' with '/', and "C:" with "C:"
        // and make sure that the path ends in a '/'
        path = normalizePath(path);
        if (!path.endsWith("/")) {
            path = path + "/";
        }
        return path;
    }

    static String exeNormalize(String exePath) {
        String path = configFile.getProperty(exePath);
        if (path == null) {
            throw new ExecutionError("C11: " + exePath + " is not set in config.properties file");
        }
        // path should have no blanks and should end in "/"
        return normalizePath(path);
    }

    public static void error(Exception e) {
        error(e.getMessage());
    }

    public static void error(String msg) {
        System.err.println(msg);
        throw new ExecutionError("C12: " + msg);
    }
    // interpreter starts with name of a prolog file that defines the 
    // category to execute
    private static List<Map<String, String>> domainS, arrowS, arrowtypeS, pathS;

    public static void main(String... args) throws ExecutionError, ConformanceError {
        String action = null;
        CTO o = null;
        int i;

        // Step 1: is this a syntactally legal invocation?  
        if (args.length == 0) {
            marquee();
            return;
        }

        // Step 2: initialize static structures and read in cat.lite.pl
        initDictionaries();

        // Step 3: load the files arguments into Var.
        //         if it is not a legitimate file, an error is reported
        //         first non-filie argument is the action
        for (i = 0; i < args.length; i++) {
            action = args[i];
            if (action.contains(".")) {
                o = Var.put(action);
            } else {
                break;
            }
        }

        // Step 4: first argument (index=0) is a file. load it into Var
        //         if it is not a legitmate file, an error will be reported
        doAction(action, args, i, o);
    }

    static public void initDictionaries() {

        readConfigFile();

        // Step 1: initialize all static structures
        StateStack.init();

        // Step 2: read in the prolog file (catlite.pl) and convert its contents
        //         into object instances that we can manipulate and execute
        readCatSpec(METAMODEL_PATH + "cat.lite.pl", System.err);
    }

    static public void doAction(String args[]) {
        doAction("run", args, -1, null);
    }

    static private void doAction(String action, String args[], int indexOfAction, CTO soleInput) {
        AorP a;

        switch (action) {
            case "run":    // <file> <file>... run arrow1 arrow2 ... arrown
                for (int j = indexOfAction + 1; j < args.length; j++) {
                    a = AorP.get(args[j]);
                    if (a == null) {
                        // not defined -- must be an rename or move
                        a = ArrowPath.returnAdHocArrow(args[j], "in run argument");
                    }
                    a.execute();
                }
                break;
                
            case "convert2":  // <file> convert to domain
                errorCount = 0;
                if (args.length != 3) {
                    marquee();
                    return;
                }
                String toDomain = args[2];
                if (Domain.get(toDomain) == null) {
                    System.err.format("domain %s does not exist\n", toDomain);
                    errorCount++;
                }
                AorP fromHub = doesArrowExist("hub2" + toDomain);
                AorP sdbS  = ArrowPath.returnAdHocArrow("sdb+S", "C77: should not happen");
                AorP toHub = doesArrowExist(soleInput.dom() + "2hub");
                if (errorCount != 0) {
                    marquee();
                    throw new ExecutionError("Errors in Conversion");
                }
                toHub.execute();
                sdbS.execute();
                fromHub.execute();

                break;
                
            case "conform":
                String cmf = conformMsgFile();
                try {
                soleInput.conform();
                } catch (ConformanceError ce) {
                    System.err.format("Conformance Error -- see %s file (listed below)", cmf);
                }
                DisplayFile.name(cmf);
                break;
                
            case "setMetaModel":
                break;
                
            default:
                marquee();
                break;
        }
    }

    static AorP doesArrowExist(String arrowName) {
        AorP arrow = AorP.get(arrowName);
        if (arrow == null) {
            System.err.format("path %s is undefined\n", arrowName);
            errorCount++;
        }
        return arrow;
    }

    public static void marquee() {
        File current = new File(".");
        //System.err.println(GetAbsolutePath(current));

        System.err.println("Usage: CatCore.Main <files> <action> <optional-extras>");
        System.err.println("       <files> is a series of one or more files that are input to the action");
        System.err.println("       action is run, convert, conform, setMetaModel");
        System.err.println("           CatCore.Main <files> run <1+ arrow_or_path_name_to_execute>");
        System.err.println("           CatCore.Main <file> convert2 <domain_name>");
        System.err.println("           CatCore.Main <file> conform");
        System.err.println("           CatCore.Main setMetaModel");
        System.err.format("%s", "\n");
        //throw new ExecutionError("C13: incorrect call to CatLite");
    }

    public static void readCatSpec(String catSpecFileName, PrintStream eps) {
        ReadPrologDatabase pdb;
        File prologFile = null;

        // Step 0: initialize all internal tables
        StateStack.push();
        domainS = null;
        arrowS = null;
        arrowtypeS = null;
        pathS = null;

        // Step 1: is it semantically legal -- lets read in the database.
        //         ReadPrologDatabase returns only if no errors were detected
        try {
            prologFile = new File(catSpecFileName);
        } catch (Exception e) {
            String emsg = "C36: " + e.getMessage();
            eps.println(emsg);
            throw new ExecutionError(emsg);
        }
        pdb = new ReadPrologDatabase(prologFile, eps);

        // Step 3: pd has its own representation of what is in a prolog database
        //         we have to map its contents to what we expect. First, let's 
        //         populate the domainDef table
        populateDomainDefTable(pdb, eps);

        // Step 4: now populate the arrows table
        populateArrowAndArrowTypeTables(pdb, eps);

        // Step 5: now populate the path table
        populatePathTable(pdb, eps);

    }

    private static void populateDomainDefTable(ReadPrologDatabase pdb, PrintStream ps) {
        if (debug) {
            System.out.println("Domain(domName,ext).");
        }
        domainS = pdb.tables.get("domainS");
        for (Map<String, String> t : domainS) {
            String name = t.get("name");
            if (name.contains("*")) {
                continue;  // don't define explicit domains that are cross-products
            }
            String ext = t.get("ext");
            if (debug) {
                System.out.format("new domaindef(%s,%s).\n", name, ext);
            }
            new Domain(name, ext);
        }
    }

    private static void populateArrowAndArrowTypeTables(ReadPrologDatabase pdb, PrintStream eps) {
        Map<String, String> arrowTuple;

        if (debug) {
            System.out.println("Arrow(name,domain,range,exe).");
        }

        // Step 1: read the table contents directly from the prolog file
        arrowS = pdb.tables.get("arrowS");
        arrowtypeS = pdb.tables.get("arrowtypeS");

        // Step 2: foreach tuple in arrowtypes table...
        for (Map<String, String> t : arrowtypeS) {

            // Step 2.1: read name, type, and exe of each arrow
            String arrowName = t.get("name");
            String type = t.get("type");
            String exe = t.get("exe");
            if (exe.equals("")) {
                exe = arrowName + "." + type;
            }

            // Step 2.2: we need to know the starting and ending domains
            //           for this arrow. We get this information in
            //           the arrowS relation.
            arrowTuple = findName(arrowName, arrowS, "database inconsistency -- "
                    + "tables arrowS and arrowtypeS should both have a tuple ", eps);

            // Step 2.3: get the names (not identifiers!) of the starting and ending nodes of the arrow
            String startId = arrowTuple.get("startId");
            String endId = arrowTuple.get("endId");

            // Step 2.4: convert these names into tuples of the domainS table
            Map<String, String> tstart, tend;
            tstart = find(startId, domainS, "database inconsistency -- tuple absent in domainS ", eps);
            tend = find(endId, domainS, "database inconsistency -- tuple absent in domainS ", eps);

            // Step 2.5: convert these tuples into names
            String domain = tstart.get("name");
            String range = tend.get("name");

            if (domain == null || range == null) {
                String msg = "database inconsistency -- null pointer for " + startId + " or " + endId;
                eps.println(msg);
                throw new ExecutionError("C14: " + msg);
            }
            // Step 2.6: now convert into the appropriate arrow object
            switch (type) {
                case "pl":
                    if (debug) {
                        System.out.format("new ArrowProlog(%s,%s,%s,%s).\n", arrowName, domain, range, exe);
                    }
                    new ArrowProlog(arrowName, domain, range, exe);
                    continue;
                case "vm":
                    if (debug) {
                        System.out.format("new ArrowVm2t(%s,%s,%s,%s).\n", arrowName, domain, range, exe);
                    }
                    new ArrowVm(arrowName, domain, range, exe);
                    continue;
                case "java":
                    if (debug) {
                        System.out.format("new ArrowJava(%s,%s,%s,%s).\n", arrowName, domain, range, exe);
                    }
                    new ArrowJava(arrowName, domain, range, exe);
                    continue;
                default:
                    error("unrecognizable arrow type " + type);
                    break;
            }
        }
    }

    private static void populatePathTable(ReadPrologDatabase pdb, PrintStream eps) {
        ArrowPath p;

        // Step 1: get the pathS table; exit if there is no such table
        pathS = pdb.tables.get("pathS");
        if (pathS == null) {
            return;
        }

        // Step 2: foreach tuple in paths table...
        for (Map<String, String> t : pathS) {
            String pathstring = t.get("path");
            p = ArrowPath.make(pathstring);
            if (debug) {
                System.out.format("new ArrowMove(%s)\n", p.toString().replace(" = ", ",").replace(";", ""));
            }
        }
    }

    static Map<String, String> find(String id, List<Map<String, String>> table, String errMsg, PrintStream eps) {
        for (Map<String, String> t : table) {
            String tid = t.get("id");
            if (tid.equals(id)) {
                return t;
            }
        }
        eps.println(errMsg + id);
        throw new ExecutionError("C15: " + errMsg + id);
    }

    static Map<String, String> findName(String name, List<Map<String, String>> table, String errMsg, PrintStream eps) {
        String tname;

        for (Map<String, String> t : table) {

            tname = t.get("name");
            //System.out.println("+++"+tname+" "+name + " " + tname.length() + " "+ name.length());
            if (tname.equals(name)) {
                return t;
            }
        }
        eps.println(errMsg + name);
        throw new ExecutionError("C16: " + errMsg + name);
    }
}
