package CatCore.vm2t;

import CatCore.ExecutionError;
import CatCore.ReadPrologDatabase;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.PrintStream;
import java.io.StringReader;
import java.io.StringWriter;
import java.lang.reflect.Constructor;
import java.util.Properties;
import org.apache.velocity.Template;
import org.apache.velocity.VelocityContext;
import org.apache.velocity.app.Velocity;

public class Main {

    static String prologFileName;
    static String templateFileName;
    static String outputFileName;
    static String contextGenClassName = null;
    static PrintStream eps;

    public static void marquee() {
        System.err.println("Usage: vm2t.Main prolog-file template-file [output-file] [-cg ContextGeneratorClass]");
        eps.println("Usage: [-cg ContextGeneratorClass] prolog-file template-file");
    }

    public static void main(String... args) {
        prologFileName = null;
        templateFileName = null;
        outputFileName = null;
        contextGenClassName = null;
        eps = null;

        // Step 0: create error file
        try {
            String emf = CatCore.Main.errMsgFile;
            eps = new PrintStream(emf);
        } catch (FileNotFoundException e) {
            e.printStackTrace();
            return;
        }

        // Step 1: read the command line arguments
        switch (args.length) {
            case 2: /* prolog,template */

                prologFileName = args[0];
                templateFileName = args[1];
                break;
            case 3: /* prolog, template, out */

                prologFileName = args[0];
                templateFileName = args[1];
                outputFileName = args[2];
                break;
            case 4: /* prolog, template, cg, context */

                if (args[2].equals("-cg")) {
                    marquee();
                    return;
                }
                prologFileName = args[0];
                templateFileName = args[1];
                contextGenClassName = args[3];
                break;
            case 5: /* prolog, template, out, cg, context */

                if (args[3].equals("-cg")) {
                    marquee();
                    return;
                }
                prologFileName = args[0];
                templateFileName = args[1];
                outputFileName = args[2];
                contextGenClassName = args[4];
                break;
            default:
                marquee();
                return;
        }
//        if (args.length > 1) {
//            if (args.length == 2) {
//                prologFileName = args[0];
//                templateFileName = args[1];
//            } else if (args.length == 4) {
//                if (args[0].equals("-cg")) {
//                    contextGenClassName = args[1];
//                } else {
//                    System.out.println("Usage: [-cg ContextGeneratorClass] prolog-file template-file");
//                    eps.println("Usage: [-cg ContextGeneratorClass] prolog-file template-file");
//                    return;
//                }
//                prologFileName = args[2];
//                templateFileName = args[3];
//            } else {
//                System.out.println("Usage: [-cg ContextGeneratorClass] prolog-file template-file");
//                eps.println("Usage: [-cg ContextGeneratorClass] prolog-file template-file");
//                return;
//            }
//        }

        // Step 2: now execute the template file
        try {
            File prologFile = new File(prologFileName);
            // the next line fixes a problem with Velocity -- it strips off white space from lines
            // that have # in them.  this allows sane indenting
            File tfn = new File(templateFileName);
            File templateFile = TemplateFile.cutWhiteSpace(tfn);

            if (!prologFile.exists()) {
                System.out.println("see " + CatCore.Main.errorMsgFile());
                eps.println("prolog-file does not exist: " + prologFileName);
                templateFile.delete();
                return;
            }

            if (!templateFile.exists()) {
                System.out.println("see " + CatCore.Main.errorMsgFile());
                eps.println("template-file does not exist.");
                templateFile.delete();
                return;
            }

            Properties p = new Properties();
            String templatePath = ".";
            if (templateFile.getParentFile() != null) {
                templatePath = templateFile.getParentFile().getCanonicalPath();
            }
            p.setProperty("file.resource.loader.path", templatePath);
            Velocity.setProperty(Velocity.RUNTIME_LOG_LOGSYSTEM, new CustomLog());
            Velocity.init(p);

            ReadPrologDatabase m = new ReadPrologDatabase(prologFile, System.err);

            VelocityContext context;
            if (contextGenClassName != null) {
                @SuppressWarnings("unchecked")
                Class<ContextGenerator> cgClass = (Class<ContextGenerator>) Class.forName(contextGenClassName);
                Constructor<ContextGenerator> cgCons = cgClass.getConstructor();
                ContextGenerator cgIns = cgCons.newInstance();
                context = cgIns.generateContext(m, outputFileName);
            } else {
                context = new DefaultContextGenerator().generateContext(m, outputFileName);
            }

            Template template;
            template = Velocity.getTemplate(templateFile.getName());
            StringWriter writer = new StringWriter();

            template.merge(context, writer);
            if (context.get("MARKER") == null) {
                System.out.println(writer.toString());
            } else {
                String deldir = null;
                Object o = context.get("DELDIR");
                if (o != null) {
                    deldir = o.toString();
                }
                splitter(writer.toString(), context.get("MARKER").toString(), deldir, eps);
            }

            templateFile.delete();
        } catch (ClassNotFoundException e) {
            System.out.println("see " + CatCore.Main.errorMsgFile());
            eps.println("The context generator class not found. Please make "
                    + "sure it is in the java classpath.");
        } catch (Exception e) {
            System.err.println("see " + CatCore.Main.errorMsgFile());
            System.err.println(e.getMessage());
            eps.println(e.getMessage());
        }

    }

    public static void splitter(String content, String marker, String deldir, PrintStream eps) {
        BufferedReader br = new BufferedReader(new StringReader(content));
        String line;
        PrintStream out = null;
        String filename, dirname;

//        System.out.println(content);
//        System.out.format("Marker = %s    Deldir = %s\n",marker,deldir);
        try {
            while ((line = br.readLine()) != null) {
                if (line.startsWith(marker)) {
                    if (out != null) {
                        out.close();
                    }
                    filename = line.substring(marker.length());
                    File f = new File(filename);
                    if (f.getParentFile() != null && !f.getParentFile().exists()) {
                        f.getParentFile().mkdirs();
                    }
                    out = new PrintStream(f);
                } else if (deldir != null && line.startsWith(deldir)) {
                    dirname = line.substring(deldir.length());
//                    System.out.println("dirname = "+ dirname);
                    File f = new File(dirname);
                    if (f.exists() && f.isDirectory()) {
                        recursiveDelete(f);
                    }
                } else {
                    if (out != null) {
                        out.println(line);
                    }
                }
            }
        } catch (IOException e) {
            e.printStackTrace(eps);
            eps.close();
            if (out != null) {
                out.close();
            }
            throw new ExecutionError("vm2t terminated");
        }
        if (out != null) {  // lines added because the last file was never closed
            out.close();
        }
    }

    // modified from http://www.mkyong.com/java/how-to-recursiveDelete-directory-in-java/
    public static void recursiveDelete(File file) throws IOException {

        if (file.isDirectory()) {
            //directory is empty, then recursiveDelete it
            if (file.list().length == 0) {
                file.delete();
            } else {
                //list all the directory contents
                String files[] = file.list();
                for (String temp : files) {
                    //construct the file structure
                    File fileDelete = new File(file, temp);
                    //recursive recursiveDelete
                    recursiveDelete(fileDelete);
                }

                //check the directory again, if empty then recursiveDelete it
                if (file.list().length == 0) {
                    file.delete();
                }
            }
        } else {
            //if file, then recursiveDelete it
            file.delete();
        }
    }
}
