/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package CatLite;

import CatCore.Main;
import CatCore.ReadPrologDatabase;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.PrintStream;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.List;
import java.util.Map;

/**
 *
 * @author dsb
 */
public class Construct {
    // this program should executed at the very end.  it assembles all of the
    // pieces created by user, and produces a metmodel directory with everything in it.
    // pieces that are missing are reported in error.txt, which can be used by advisement programs

    static PrintStream eps = null;

    public static void main(String[] args) {
        ReadPrologDatabase pdb;
        File prologFile, mmdir;
        String mmDirPath;
        String catLitePl, netbeansAppName, modelPath;
        PrintStream rmf;

        // Step 0: do the usual marquee testing
        if (args.length != 2) {
            marquee();
            return;
        }
        
        // Step 1: give names to these inputs -- too easy to forget their bindings
        catLitePl  = args[0];
        netbeansAppName = args[1].substring(args[1].lastIndexOf("/")+1).replace(".metamodel.mm",""); 
        modelPath = args[1].replace(netbeansAppName+".metamodel.mm", "");
       // current directory
        try {
            eps = new PrintStream(CatCore.Main.errMsgFile);
        } catch (FileNotFoundException e) {
            System.err.format("cannot open error message file %s\n", CatCore.Main.errMsgFile);
            return;
        }

        // Step 2: read in the metamodel-to-be file
        try {
            prologFile = new File(catLitePl);
        } catch (Exception e) {
            eps.format("Cannot open file %s\n", catLitePl);
            return;
        }
        pdb = new ReadPrologDatabase(prologFile, eps);

        // Step 3: create the metamodel directory
        try {
            mmdir = new File(modelPath + netbeansAppName);
            if (mmdir.exists()) {
                delete(mmdir,eps);
            }
            mmdir.mkdir();
        } catch (IOException e) {
            eps.format("Cannot (re-)create directory %s\n",netbeansAppName);
            return;
        }
        
        // Step 4: initialize variables
        //         mmDirPath -- this is the directory that will contain the metamodel-assembled artifacts
        //         mmodelPath -- this is the current directory (the parent of mmDirPath).
        //         mmName     -- a CTO has full name (name.domain.ext).  mmname is the unqualified first part of this name
        mmDirPath = CatCore.Main.normalizePath(CatCore.Main.GetAbsolutePath(mmdir));
        if (!mmDirPath.endsWith("/")) {
            mmDirPath = mmDirPath + "/";
        }

        // Step 5: now populate the created directory with files 
        try {
            // Step 5a: start with the metamodel and its beautiful violet representation,
            //          giving both standardized names in the process
            copyfile(catLitePl, mmDirPath + "cat.lite.pl");
            copyfile(modelPath + netbeansAppName + ".state.violet", mmDirPath + "cat.state.violet");

//            // Step 4b: now add the rubberstamped source files to src/${mmName}/
//            //          both directories should exist in a netbeans project
//            //          these files will need to be compiled
//            mkdir(mmDirPath, "src");
//            String srcdir = mmDirPath+"src/";
//            mkdir(srcdir,netbeansAppName);
//            srcdir = srcdir + netbeansAppName +"/";
//            mkStandardFile(srcdir, netbeansAppName, "Main");
//            mkStandardFile(srcdir, netbeansAppName, "Gui");
//            mkStandardFile(srcdir, netbeansAppName, "Validate");
//            mkStandardFile(srcdir, netbeansAppName, "Clean");
//            

            // Step 4c: now the empty libvm, and libpl directories
            mkdir(mmDirPath, "libvm"); 
            mkdir(mmDirPath, "libpl");

            // Step 4d: create a read me file on which to append instructions
            rmf = readMeFile(mmDirPath,netbeansAppName);
            
            // Step 4e: now copy all of the schema and (dummy) conform files
            String domainName, domainExt,inputFile;
            int counter = 1;
            
            List<Map<String, String>> domainTable = pdb.tables.get("domainS");
            for(Map<String,String> tuple : domainTable) {
                domainName = tuple.get("name");
                if (domainName.contains("*"))   // ignore cross products
                    continue;
                domainExt = tuple.get("ext");
                if (!domainExt.equals("pl"))
                    continue;
                inputFile = domainName + ".schema.pl";
                copyfile(modelPath+inputFile, mmDirPath+"libpl/"+inputFile);
                mkEmptyConformFile(mmDirPath, domainName);
                rmf.format("%d) rewrite the dummy libpl/%s.conform.pl\n", counter++, domainName);
            }
            
            // Step 4f: now for each prolog and vm arrow, these files don't
            // yet exist.  So we will place dummy file for each in their 
            // repective directories
            
            List<Map<String, String>> arrowTypeTable = pdb.tables.get("arrowtypeS");
            for (Map<String, String> tuple : arrowTypeTable) {
//                String arrowName = tuple.get("name");
                String type = tuple.get("type");
                String exe  = tuple.get("exe");
                switch (type) {
                    case "java": 
                        rmf.format("%d) write java application %s and place it on your CLASSPATH\n",counter++, exe);
                        continue;
                    case "pl":
                        rmf.format("%d) rewrite the dummy prolog (M2M) transformation file libpl/%s\n", counter++, exe);
                        mkDummyPlFile(mmDirPath+"libpl/", exe);
                        continue;
                    case "vm":
                        rmf.format("%d) rewrite the dummy velocity (M2T) file libvm/%s\n", counter++, exe);
                        mkDummyVmFile(mmDirPath+"libvm/", exe);
                        continue;
                    case "exe":
                        System.err.println("NOT IMPLEMENTED YET");
                }
                
            }
            String catlib = System.getenv("CATLIB");
            rmf.format("%d) post this completed directory in %s for others to use\n",counter++,Main.normalizePath(catlib));
            rmf.format("%d) optionally, write an advice program to make your tool easier for others to use\n",counter++);
            rmf.close();
        } catch (Exception e) {
            eps.println(e.getMessage());
        }
        eps.close();
    }

    public static void marquee() {
        System.err.println("Usage: Construct <name>.lite.pl <name>.mm");
        System.err.println("       same <name> required in both places");
    }

    // modified from http://www.mkyong.com/java/how-to-delete-directory-in-java/
    public static void delete(File file, PrintStream eps) throws IOException {
        try {
        if (file.isDirectory()) {
            //directory is empty, then delete it
            if (file.list().length == 0) {
                file.delete();
            } else {
                //list all the directory contents
                String files[] = file.list();
                for (String temp : files) {
                    //construct the file structure
                    File fileDelete = new File(file, temp);
                    //recursive delete
                    delete(fileDelete,eps);
                }

                //check the directory again, if empty then delete it
                if (file.list().length == 0) {
                    file.delete();
                }
            }
        } else {
            //if file, then delete it
            file.delete();
        }
        }
        catch (IOException e) {
            eps.println(e.getMessage());
        }
    }

    static void copyfile(String from, String to) {
        try {
            File ffrom = new File(from);
            File fto = new File(to);
            Path pfrom = ffrom.toPath();
            Path pto = fto.toPath();
            Files.copy(pfrom, pto);
        } catch (IOException e) {
            eps.println("cannot copy file " + from);
        }
    }

    static File mkdir(String mmname, String dir) {
        File result = new File(mmname + dir);
        if (result.exists())
            return result;
        result.mkdir();
        return result;
    }

//    static void mkStandardFile(String inDirectory, String packageName, String className) {
//        String fileName = inDirectory + className + ".java";
//        try {
//            PrintStream ps = new PrintStream(fileName);
//            ps.format("package %s;\n\n", packageName);
//            ps.format("public class %s {\n\n", className);
//            ps.format("   public static void main(String... args) {\n", (Object) null);
//            ps.format("      CatCore.%s.main(new %s().getClass(), args);\n", className, className);
//            ps.format("   }\n");
//            ps.format("}\n");
//            ps.close();
//        } catch (Exception e) {
//            eps.format("cannot create %s\n", fileName);
//        }
//    }
    
    static void mkDummyPlFile(String inDirectory, String fileName) {
        try {
            PrintStream ps = new PrintStream(inDirectory+fileName);
//            ps.format("/* this is a bogus file that stands in place of a real implementation for %s\n",fileName);
            ps.format("run:- write('please write a proper %s file').", fileName);
            ps.close();
        } catch (FileNotFoundException e) {
            eps.format("cannot create %s\n", inDirectory+fileName);
        }
    }
    
    static void mkDummyVmFile(String inDirectory, String fileName) {
        try {
            PrintStream ps = new PrintStream(inDirectory+fileName);
            ps.format("#set($MARKER=\"//----\")\n");
            ps.format("${MARKER}${OutputFileName}\n");
            ps.format("You forgot to write this\n");
            ps.format("velocity template %s!\n",fileName);
            ps.close();
        } catch (FileNotFoundException e) {
            eps.format("cannot create %s\n", inDirectory+fileName);
        }
    }
    
    static void mkEmptyConformFile(String mmDirPath, String domainName) {
        try {
            String fileName = mmDirPath + "libpl/" + domainName + ".conform.pl";
            PrintStream ps = new PrintStream(fileName);
            ps.format("/* these are the conformance rules for domain %s */\n",domainName);
            ps.format("/* file currently has no rules to evaluate -- just add the rules to run */\n\n", (Object) null);
            ps.format("run:-write('you need to write a real conform file!').\n", (Object) null);
            ps.close();
        } catch (FileNotFoundException e) {
            eps.format("cannot create libpl/%s.conform.pl\n",domainName);
        }
    }
    
    static PrintStream readMeFile(String inDirectory, String mmname) {
        PrintStream rmf;
        String txt = 
            "This directory contains the core definitions of the %s metamodel.\n" +
            "It is still incomplete. Follow these instructions:\n\n";
        try {
            rmf = new PrintStream(inDirectory+"README.TXT");
            rmf.format(txt,mmname);
            return rmf;
        }
        catch (FileNotFoundException e) {
            eps.format("cannot create README.TXT file\n",(Object) null);
        }
        return null;
    }

}
